import type { TokenInjectType } from '@/api';
import { ref } from 'vue';
import { NProgress, NButton, NIcon } from 'naive-ui';
import filesize from 'filesize';
import { Archive, Download } from '@vicons/carbon';
import { FS, check_token, isErrorResponse } from '@/api';
import type { DialogApiInjection } from 'naive-ui/es/dialog/src/DialogProvider';

export default function createZipDialog(
	nodes: number[],
	dialog: DialogApiInjection,
	jwt: TokenInjectType
) {
	const progress = ref(0);
	const total = ref(1);
	const percentage = ref(0);
	const done = ref(false);
	const dia = dialog.create({
		title: 'Create Archive...',
		closable: false,
		closeOnEsc: false,
		maskClosable: false,
		icon: () => <Archive />,
		content: () => (
			<NProgress
				type="line"
				percentage={percentage.value}
				height={20}
				status="info"
				showIndicator={false}
			/>
		),
		action: () =>
			done.value ? (
				<NButton
					onClick={async () => {
						const token = await check_token(jwt);
						if (!token) return;
						if (nodes.length == 1)
							FS.download_file(token, nodes[0]);
						else FS.download_multi_file(token, nodes);
						dia.destroy();
					}}
				>
					{{
						icon: () => (
							<NIcon>
								<Download />
							</NIcon>
						),
						default: () => 'Download archive'
					}}
				</NButton>
			) : (
				<div>
					{filesize(progress.value, {
						base: 2,
						standard: 'jedec'
					})}
					/
					{filesize(total.value, {
						base: 2,
						standard: 'jedec'
					})}
					- {Math.floor(percentage.value * 1000) / 1000}%
				</div>
			)
	});
	let updateRunning = false;
	const updateInterval = setInterval(async () => {
		if (updateRunning) return;
		updateRunning = true;
		const token = await check_token(jwt);
		if (!token) return;
		const resp = await FS.create_zip(token, nodes);
		if (isErrorResponse(resp)) return;
		if (resp.done) {
			percentage.value = 100;
			clearInterval(updateInterval);
			done.value = true;
		} else {
			progress.value = resp.progress ?? 0;
			total.value = resp.total ?? 1;
			if (total.value == 0) total.value = 1;
			percentage.value = (progress.value / total.value) * 100;
		}
		updateRunning = false;
	}, 500);
	return dia;
}
